==> openclip-0.0.9.tar.gz: openclip-0.0.9/openclip/elements.py <==
# -*- coding: utf-8 -*-
"""
    openclip/elements.py
    =======================================

    provides object/class representations of each of the primary
    OpenClip XML Elements and core functionality required by each

    these classes are primarily used in the core `OpenClip` class/object
    but also have functionality on their own for some use cases
"""
import uuid
from copy import deepcopy
from . import LOG
import openclip.util as util
import openclip.constants as _c
from openclip.exceptions import *
try:
    from lxml import etree as ET
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as ET
        LOG.warning("running with cElementTree on Python 2.5+")
    except ImportError:
        try:
            import xml.etree.ElementTree as ET
            LOG.warning("running with ElementTree on Python 2.5+")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as ET
                LOG.warning("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as ET
                    LOG.warning("running with ElementTree")
                except ImportError:
                    LOG.critical("Failed to import ElementTree from any known place")


__all__ = [ 'Clip', 'Tracks', 'Track', 'Feeds', 'Feed', 'Spans', 'Span', 'Versions', 'Version' ]

__name__ = 'openclip.elements'


class OpenClipBase(object):
    """
    base class to inherit our openclip classes from
    * uses the class name determined using the tag
      property to create the base etree.Element
    * ultimately we may introduce mixins for various
      software packages?
    """

    childrenRequired=[]
    """these are single child elements that are required
    multiple child element types are added with the following list attribute
    this list is used to propagate the tree within each subclass"""
    childrenMultRequired=[]
    """because we want to handle lists of children as lists we define
    them separately"""
    childrenOptional=[]
    """optional children - not used... yet"""

    def __init__(self, element=None, parent=None):
        # store a copy of the incoming element
        # if None that's OK
        self.src_element = deepcopy(element)
        # if the element is empty let's create one
        if element is not None:
            self.element = element
        else:
            self.element = self.init_element()
        self.parent = parent
        # we have the element so now let's get the children
        self._init_children(self.element)

    @property
    def tag(self):
        # this property returns the class name in lowercase
        return type(self).__name__.lower()

    @property
    def childrenRequired(self):
        # this property returns the required children for the given element class
        fmt_str = "{className}.childrenRequired"
        eval_str= fmt_str.format(className=type(self).__name__)
        return eval(children_eval_str)

    @property
    def childrenMultRequired(self):
        # this property returns the required children for the given element class
        fmt_str = "{className}.childrenMultRequired"
        eval_str= fmt_str.format(className=type(self).__name__)
        return eval(children_eval_str)

    @property
    def childrenOptional(self):
        # this property returns the optional children for the given element class
        fmt_str = "{className}.childrenOptional"
        eval_str = fmt_str.format(className=type(self).__name__)
        return eval(children_eval_str)

    @property
    def xml(self):
        """
        this returns the object src_element element as a string
        based on the class name (using tag here)
        we set xml_header to False because we only want the header
        on our top level tree
        """
        return util.element_to_xml(self.etree,xml_header=False)

    # def __str__(self):
    #     # return the unicode string encoded properly
    #     return self.__unicode__().encode('utf8', 'replace')

    def init_element(self):
        element = ET.Element(self.tag)
        element.set('type', self.tag)
        element.set('version', _c.OPENCLIP_VERSION)
        return element

    def _init_children(self, element=None):
        if 'element' is None:
            for child in self.childrenRequired:
                fmt_str = 'self.{child_attr} = {child_class}()'
                exec_str = fmt_str.format(child_attr=child, child_class=child.capitalize())
                exec(exec_str)
                exec('self.{child_attr}.parent = self'.format(child_attr=child))
        else:
            for child in self.childrenRequired:
                if not element.find(child) is None:
                    child_element = element.find(child)
                    fmt_str = 'self.{child_attr} = {child_class}(element=child_element)'
                    exec_str = fmt_str.format(child_attr=child, child_class=child.capitalize())
                    element.remove(child_element)
                else:
                    fmt_str = 'self.{child_attr} = {child_class}()'
                    exec_str = fmt_str.format(child_attr=child, child_class=child.capitalize())
                exec(exec_str)
                exec('self.{child_attr}.parent = self'.format(child_attr=child))
            for child in self.childrenMultRequired:
                if not element.findall(child) is None:
                    for child_element in element.findall(child):
                        fmt_str = 'grandchild={child_class}(element=child_element)'
                        exec(fmt_str.format(child_class=child.capitalize()))
                        exec('grandchild.parent=self')
                        fmt_str = 'self.{child_attr}_list.append(grandchild)'
                        exec(fmt_str.format(child_attr=child))
                        element.remove(child_element)
            # self.element = element # not needed as object changes are persistent

    @property
    def etree(self):
        # property that dynamically returns the xml tree from the current element
        # descending down to it's deepest descendants
        etreeRoot = deepcopy(self.element)
        for child_type in self.childrenRequired:
            child = eval('self.{child_type}'.format(child_type=child_type))
            etreeRoot.append(child.etree)
            # exex_str = 'self.etree.append(self.{child_type}.element)'
            # exec(exec_str.format(child_type=child_type))
        for child_type in self.childrenMultRequired:
            child_list = eval('self.{child_type}_list'.format(child_type=child_type))
            for child in child_list:
                etreeRoot.append(child.etree)
        return etreeRoot


class Clip(OpenClipBase):
    """
        <clip type="clip" version="3"/>
    """

    childrenRequired = [ 'tracks', 'versions' ]
    childrenMultRequired=[]
    childrenOptional = [ 'handler', 'name', 'userData', 'comment', 'sourceName', 'startTimeCode', 'duration', 'editRate', 'dropMode' ]
    def __init__(self, element=None, parent=None):
        super(Clip, self).__init__(element)

    def _get_name(self):
        if self.element.find('name') is None:
            self.name = ET.SubElement(self.element, 'name', type='string')
        return self.element.find('name')

    def _set_name(self, value):
        if isinstance(value, ET._Element) and value.tag=="name":
            if self.element.find('name') is None:
                self.name = ET.SubElement(self.element, 'name', type='string')
            self.name.text = value.text
        elif isinstance(value, basestring):
            if self.element.find('name') is None:
                self.name = ET.SubElement(self.element, 'name', type='string')
            self.name.text = value
        else:
            self.name = ET.SubElement(self.element, 'name', type='string')
            self.name.text = self.basename

    name = property(_get_name, _set_name, doc='Name Element and Text')



class Tracks(OpenClipBase):
    """
        <tracks/>
    """

    childrenRequired = []
    childrenMultRequired=[ 'track' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.track_list=[]
        super(Tracks, self).__init__(element)

    @property
    def currentVersion(self):
        return self.get('currentVersion')

    def _get_currentVersion(self):
        return self.element.get('currentVersion')

    def _set_currentVersion(self, value):
        self.element.set(value)

    currentVersion=property(_get_currentVersion,_set_currentVersion)

    def add_track(self, track_element):
        self.append(track_element)


class Track(OpenClipBase):
    """
        <track uid="client-defined unique identifier">
            <trackType>video</trackType>
        </track>
    """

    childrenRequired = [ 'feeds' ] # 'trackType' is also required & set separately
    childrenMultRequired=[]
    childrenOptional = [ 'extendedType', 'handler', 'sourceName', 'startTimecode', 'dropMode', 'duration', 'name', 'editRate', 'userData', 'comment' ]

    def __init__(self, element=None, parent=None):
        super(Track, self).__init__(element)

    def _get_uid(self):
        track_uid = self.element.get('uid')
        return track_uid

    def _set_uid(self,value):
        self.element.set('uid', value)

    uid=property(_get_uid,_set_uid)


class Feeds(OpenClipBase):
    """
        <feeds currentVersion="the <feed> vuid">
        </feeds>
    """

    childrenRequired = []
    childrenMultRequired=[ 'feed' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.feed_list=[]
        super(Feeds, self).__init__(element)

    def _get_currentVersion(self):
        feeds_currentVersion = self.element.get('currentVersion')
        return feeds_currentVersion

    def _set_currentVersion(self,value):
        self.element.set('currentVersion',value)

    currentVersion=property(_get_currentVersion,_set_currentVersion)

    def vuid_list(self):
        return [feed.element.get('vuid') for feed in self.feed_list]


class Feed(OpenClipBase):
    """
        <feed  vuid="client defined" uid="client-defined unique identifier">
        </feed>
    """

    childrenRequired = [ 'spans' ]
    childrenMultRequired=[]
    childrenOptional = [ 'handler', 'storageFormat', 'sampleRate', 'startTimecode', 'startOffset', 'userData', 'comment' ]

    def __init__(self, element=None, parent=None):
        super(Feed, self).__init__(element)

    def _get_vuid(self):
        feed_vuid = self.element.get('vuid')
        return feed_vuid

    def _set_vuid(self,value):
        self.element.set('vuid',value)

    vuid=property(_get_vuid,_set_vuid)

    def _get_uid(self):
        feed_uid = self.element.get('uid') or 'id%s' % uuid.uuid4()
        return feed_uid

    def _set_uid(self,value):
        self.element.set('uid',value)

    uid=property(_get_uid,_set_uid)


class Spans(OpenClipBase):
    """
        <spans>
        </spans>
    """

    childrenRequired = [ ]
    childrenMultRequired=[ 'span' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.span_list = []
        super(Spans, self).__init__(element)


class Span(OpenClipBase):
    """
        <span>
            <duration>number of samples in the media sequence</duration>
            <path>path the media sequence</path>
        </span>
    """

    childrenRequired = [] # 'duration' is also REQUIRED but set elsewhere
    childrenMultRequired=[]
    childrenOptional = [ 'paths', 'path', 'userData' ]

    def __init__(self, element=None, parent=None):
        super(Span, self).__init__(element)


class Versions(OpenClipBase):
    """
        <versions currentVersion="IDREF to one of the <version> uids">
        </versions>
    """

    childrenRequired = []
    childrenMultRequired=[ 'version' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.version_list = []
        super(Versions, self).__init__(element)

    def _get_currentVersion(self):
        versions_currentVersion = self.element.get('currentVersion')
        return versions_currentVersion

    def _set_currentVersion(self,value):
        self.element.set('currentVersion',value)

    currentVersion=property(_get_currentVersion,_set_currentVersion)

    def uid_list(self):
        return [version.element.get('uid') for version in self.version_list]


class Version(OpenClipBase):
    """
        <version type="version" uid="IDREF to one of the <feed> vuids"/>
            <name>v1</name>
            <creationDate>2011/02/03 10:53:44</creationDate>
            <userData/>
            <comment/>
        </version>
    """

    childrenRequired = []
    childrenMultRequired=[]
    childrenOptional = [ 'name', 'comment', 'creationDate', 'userData' ]

    def __init__(self, element=None, parent=None):
        super(Version, self).__init__(element)

    def _get_uid(self):
        track_uid = self.element.get('uid')
        return track_uid

    def _set_uid(self,value):
        self.element.set('uid', value)

    uid=property(_get_uid,_set_uid)

==> openclip-0.0.10.tar.gz: openclip-0.0.10/openclip/elements.py <==
# -*- coding: utf-8 -*-
""" The ``openclip.elements`` module provides all of the main OpenClip XML Element classes.

Each element class (Clip, Tracks, Track, Versions, Version, Feeds, Feed, Spans, Span) has
individual requirements. This module provides object/class representations of each of the primary
OpenClip XML Elements and core functionality required by each.

Note:
    these classes are primarily used in the core `OpenClip` class/object
    but also have functionality on their own for some use cases

"""
import uuid
from copy import deepcopy
from . import LOG
import openclip.util as util
import openclip.constants as _c
from openclip.exceptions import *
try:
    from lxml import etree as ET
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as ET
        LOG.warning("running with cElementTree on Python 2.5+")
    except ImportError:
        try:
            import xml.etree.ElementTree as ET
            LOG.warning("running with ElementTree on Python 2.5+")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as ET
                LOG.warning("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as ET
                    LOG.warning("running with ElementTree")
                except ImportError:
                    LOG.critical("Failed to import ElementTree from any known place")


__all__ = [ 'OpenClipBase', 'Clip', 'Tracks', 'Track', 'Feeds', 'Feed', 'Spans', 'Span', 'Versions', 'Version' ]

__name__ = 'openclip.elements'


class OpenClipBase(object):
    """ base class to inherit our openclip classes from

    - uses the class name determined using the tag property to create the base etree.Element

    todo:
        - ultimately we may introduce mixins for various software packages?
    """

    childrenRequired=[]
    """these are single child elements that are required
    multiple child element types are added with the following list attribute
    this list is used to propagate the tree within each subclass"""
    childrenMultRequired=[]
    """because we want to handle lists of children as lists we define
    them separately"""
    childrenOptional=[]
    """optional children - not used... yet"""

    def __init__(self, element=None, parent=None):
        # store a copy of the incoming element
        # if None that's OK
        self.src_element = deepcopy(element)
        # if the element is empty let's create one
        if element is not None:
            self.element = element
        else:
            self.element = self.init_element()
        self.parent = parent
        # we have the element so now let's get the children
        self._init_children(self.element)

    @property
    def tag(self):
        # this property returns the class name in lowercase
        return type(self).__name__.lower()

    @property
    def childrenRequired(self):
        # this property returns the required children for the given element class
        fmt_str = "{className}.childrenRequired"
        eval_str= fmt_str.format(className=type(self).__name__)
        return eval(children_eval_str)

    @property
    def childrenMultRequired(self):
        # this property returns the required children for the given element class
        fmt_str = "{className}.childrenMultRequired"
        eval_str= fmt_str.format(className=type(self).__name__)
        return eval(children_eval_str)

    @property
    def childrenOptional(self):
        # this property returns the optional children for the given element class
        fmt_str = "{className}.childrenOptional"
        eval_str = fmt_str.format(className=type(self).__name__)
        return eval(children_eval_str)

    @property
    def xml(self):
        """
        this returns the object src_element element as a string
        based on the class name (using tag here)
        we set xml_header to False because we only want the header
        on our top level tree
        """
        return util.element_to_xml(self.etree,xml_header=False)

    # def __str__(self):
    #     # return the unicode string encoded properly
    #     return self.__unicode__().encode('utf8', 'replace')

    def init_element(self):
        element = ET.Element(self.tag)
        element.set('type', self.tag)
        element.set('version', _c.OPENCLIP_VERSION)
        return element

    def _init_children(self, element=None):
        if 'element' is None:
            for child in self.childrenRequired:
                fmt_str = 'self.{child_attr} = {child_class}()'
                exec_str = fmt_str.format(child_attr=child, child_class=child.capitalize())
                exec(exec_str)
                exec('self.{child_attr}.parent = self'.format(child_attr=child))
        else:
            for child in self.childrenRequired:
                if not element.find(child) is None:
                    child_element = element.find(child)
                    fmt_str = 'self.{child_attr} = {child_class}(element=child_element)'
                    exec_str = fmt_str.format(child_attr=child, child_class=child.capitalize())
                    element.remove(child_element)
                else:
                    fmt_str = 'self.{child_attr} = {child_class}()'
                    exec_str = fmt_str.format(child_attr=child, child_class=child.capitalize())
                exec(exec_str)
                exec('self.{child_attr}.parent = self'.format(child_attr=child))
            for child in self.childrenMultRequired:
                if not element.findall(child) is None:
                    for child_element in element.findall(child):
                        fmt_str = 'grandchild={child_class}(element=child_element)'
                        exec(fmt_str.format(child_class=child.capitalize()))
                        exec('grandchild.parent=self')
                        fmt_str = 'self.{child_attr}_list.append(grandchild)'
                        exec(fmt_str.format(child_attr=child))
                        element.remove(child_element)
            # self.element = element # not needed as object changes are persistent

    @property
    def etree(self):
        # property that dynamically returns the xml tree from the current element
        # descending down to it's deepest descendants
        etreeRoot = deepcopy(self.element)
        for child_type in self.childrenRequired:
            child = eval('self.{child_type}'.format(child_type=child_type))
            etreeRoot.append(child.etree)
            # exex_str = 'self.etree.append(self.{child_type}.element)'
            # exec(exec_str.format(child_type=child_type))
        for child_type in self.childrenMultRequired:
            child_list = eval('self.{child_type}_list'.format(child_type=child_type))
            for child in child_list:
                etreeRoot.append(child.etree)
        return etreeRoot


class Clip(OpenClipBase):
    """
    Example:
        <clip type="clip" version="3">
        </clip>
    """

    childrenRequired = [ 'tracks', 'versions' ]
    childrenMultRequired=[]
    childrenOptional = [ 'handler', 'name', 'userData', 'comment', 'sourceName', 'startTimeCode', 'duration', 'editRate', 'dropMode' ]
    def __init__(self, element=None, parent=None):
        super(Clip, self).__init__(element)

    def _get_name(self):
        if self.element.find('name') is None:
            self.name = ET.SubElement(self.element, 'name', type='string')
        return self.element.find('name')

    def _set_name(self, value):
        if isinstance(value, ET._Element) and value.tag=="name":
            if self.element.find('name') is None:
                self.name = ET.SubElement(self.element, 'name', type='string')
            self.name.text = value.text
        elif isinstance(value, basestring):
            if self.element.find('name') is None:
                self.name = ET.SubElement(self.element, 'name', type='string')
            self.name.text = value
        else:
            self.name = ET.SubElement(self.element, 'name', type='string')
            self.name.text = self.basename

    name = property(_get_name, _set_name, doc='Name Element and Text')



class Tracks(OpenClipBase):
    """
        <tracks/>
    """

    childrenRequired = []
    childrenMultRequired=[ 'track' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.track_list=[]
        super(Tracks, self).__init__(element)

    @property
    def currentVersion(self):
        return self.get('currentVersion')

    def _get_currentVersion(self):
        return self.element.get('currentVersion')

    def _set_currentVersion(self, value):
        self.element.set(value)

    currentVersion=property(_get_currentVersion,_set_currentVersion)

    def add_track(self, track_element):
        self.append(track_element)


class Track(OpenClipBase):
    """
        <track uid="client-defined unique identifier">
            <trackType>video</trackType>
        </track>
    """

    childrenRequired = [ 'feeds' ] # 'trackType' is also required & set separately
    childrenMultRequired=[]
    childrenOptional = [ 'extendedType', 'handler', 'sourceName', 'startTimecode', 'dropMode', 'duration', 'name', 'editRate', 'userData', 'comment' ]

    def __init__(self, element=None, parent=None):
        super(Track, self).__init__(element)

    def _get_uid(self):
        track_uid = self.element.get('uid')
        return track_uid

    def _set_uid(self,value):
        self.element.set('uid', value)

    uid=property(_get_uid,_set_uid)


class Feeds(OpenClipBase):
    """
        <feeds currentVersion="the <feed> vuid">
        </feeds>
    """

    childrenRequired = []
    childrenMultRequired=[ 'feed' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.feed_list=[]
        super(Feeds, self).__init__(element)

    def _get_currentVersion(self):
        feeds_currentVersion = self.element.get('currentVersion')
        return feeds_currentVersion

    def _set_currentVersion(self,value):
        self.element.set('currentVersion',value)

    currentVersion=property(_get_currentVersion,_set_currentVersion)

    def vuid_list(self):
        return [feed.element.get('vuid') for feed in self.feed_list]


class Feed(OpenClipBase):
    """
        <feed  vuid="client defined" uid="client-defined unique identifier">
        </feed>
    """

    childrenRequired = [ 'spans' ]
    childrenMultRequired=[]
    childrenOptional = [ 'handler', 'storageFormat', 'sampleRate', 'startTimecode', 'startOffset', 'userData', 'comment' ]

    def __init__(self, element=None, parent=None):
        super(Feed, self).__init__(element)

    def _get_vuid(self):
        feed_vuid = self.element.get('vuid')
        return feed_vuid

    def _set_vuid(self,value):
        self.element.set('vuid',value)

    vuid=property(_get_vuid,_set_vuid)

    def _get_uid(self):
        feed_uid = self.element.get('uid') or 'id%s' % uuid.uuid4()
        return feed_uid

    def _set_uid(self,value):
        self.element.set('uid',value)

    uid=property(_get_uid,_set_uid)


class Spans(OpenClipBase):
    """
        <spans>
        </spans>
    """

    childrenRequired = [ ]
    childrenMultRequired=[ 'span' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.span_list = []
        super(Spans, self).__init__(element)


class Span(OpenClipBase):
    """
        <span>
            <duration>number of samples in the media sequence</duration>
            <path>path the media sequence</path>
        </span>
    """

    childrenRequired = [] # 'duration' is also REQUIRED but set elsewhere
    childrenMultRequired=[]
    childrenOptional = [ 'paths', 'path', 'userData' ]

    def __init__(self, element=None, parent=None):
        super(Span, self).__init__(element)


class Versions(OpenClipBase):
    """
        <versions currentVersion="IDREF to one of the <version> uids">
        </versions>
    """

    childrenRequired = []
    childrenMultRequired=[ 'version' ]
    childrenOptional = []

    def __init__(self, element=None, parent=None):
        self.version_list = []
        super(Versions, self).__init__(element)

    def _get_currentVersion(self):
        versions_currentVersion = self.element.get('currentVersion')
        return versions_currentVersion

    def _set_currentVersion(self,value):
        self.element.set('currentVersion',value)

    currentVersion=property(_get_currentVersion,_set_currentVersion)

    def uid_list(self):
        return [version.element.get('uid') for version in self.version_list]


class Version(OpenClipBase):
    """
        <version type="version" uid="IDREF to one of the <feed> vuids"/>
            <name>v1</name>
            <creationDate>2011/02/03 10:53:44</creationDate>
            <userData/>
            <comment/>
        </version>
    """

    childrenRequired = []
    childrenMultRequired=[]
    childrenOptional = [ 'name', 'comment', 'creationDate', 'userData' ]

    def __init__(self, element=None, parent=None):
        super(Version, self).__init__(element)

    def _get_uid(self):
        track_uid = self.element.get('uid')
        return track_uid

    def _set_uid(self,value):
        self.element.set('uid', value)

    uid=property(_get_uid,_set_uid)
